/*
 * Main module
 * Copyright 2017 William Lupshenko
 * 
 * This file is a part of TUServer
 * 
 * TUServer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the license, or
 * (at your option) any later version
 */

var
	path = require("path"),
	fs = require("fs"),
	crypto = require("crypto"),
	readline = require("readline");

process.chdir(path.dirname(require.main.filename));

var app = {
	//Secondary variables
		sessions: {},
		sessionStatuses: {},
		
		readlineInterface: readline.createInterface({
			input: process.stdin,
			output: process.stdout,
			
			prompt: ""
		}),
		
		socket: require("./socket.js"),
		gameWorld: require("./gameWorld.js"),
	
	//Methods
		run: function(argv) {
			console.log(
				"Starting server..."
			);
			
			process.stdout.write(
				"Adding events... "
			);
			
			this.readlineInterface.on("SIGINT", function() {
				console.log(
					"Shutting down server..."
				);
				
				for (var i in app.sessions) {
					app.socket.emit("disconnection", {
						login: i,
						sessionId: app.sessions[i]
					});
				}
				
				process.stdout.write(
					"Autosaving... "
				);
				
				game.config.write("gameWorld");
				game.config.write("players");
				
				console.log(
					"DONE"
				);
				
				console.log(
					"Server died"
				);
				
				process.exit(0);
			});
			
			this.socket.on("connection", function(data) {
				var sessionId = crypto.randomBytes(16).toString("hex");
				
				if (data.login in app.sessions) {
					console.log(
						sessionId +
						" failed to log in as " +
						data.login
					);
					
					return {
						isAllowed: false,
						reason: "this player already playing"
					};
				} else {
					app.sessions[data.login] = sessionId;
					app.sessionStatuses[data.login] = true;
					
					app.gameWorld.players.connect(data.login);
					
					console.log(
						sessionId +
						" logged in as " +
						data.login
					);
					
					return {
						isAllowed: true,
						sessionId: sessionId
					};
				}
			});
			
			this.socket.on("ping", function(data) {
				if (app.sessions[data.login] == data.sessionId) {
					app.sessionStatuses[data.login] = true;
					
					return {};
				} else if (!(data.login in app.sessions)) {
					return {
						error: "player kicked"
					};
				} else {
					return {
						error: "invalid session id"
					};
				}
			});
			
			this.socket.on("rendering", function(data) {
				if (app.sessions[data.login] == data.sessionId) {
					return {
						gfx: app.gameWorld.render(
							data.login
						),
						sounds: app.gameWorld.players.sounds[
							data.login
						].splice(0,  app.gameWorld.players.sounds[
							data.login
						].length)
					};
				} else if (!(data.login in app.sessions)) {
					return {
						error: "player kicked"
					};
				} else {
					return {
						error: "invalid session id"
					};
				}
			});
			
			this.socket.on("keyPress", function(data) {
				if (app.sessions[data.login] == data.sessionId) {
					if (
						data.keyCode == game.config.list.main.hotkeys.menu &&
						
						app.gameWorld.players.list[data.login].currentGuiType == "default"
					) {
						app.socket.emit("disconnection", {
							login: data.login,
							sessionId: data.sessionId
						});
					} else {
						app.gameWorld.handleHotkey(
							data.login,
							
							data.keyCode,
							data.isDown
						);
					}
					
					return {};
				} else if (!(data.login in app.sessions)) {
					return {
						error: "player kicked"
					};
				} else {
					return {
						error: "invalid session id"
					};
				}
			});
			
			this.socket.on("disconnection", function(data) {
				if (app.sessions[data.login] == data.sessionId) {
					delete app.sessions[data.login];
					delete app.sessionStatuses[data.login];
					
					app.gameWorld.players.disconnect(data.login);
					
					console.log(
						data.sessionId +
						" logged out as " +
						data.login
					);
					
					return {};
				} else if (!(data.login in app.sessions)) {
					return {
						error: "player kicked"
					};
				} else {
					return {
						error: "invalid session id"
					};
				}
			});
			
			console.log(
				"DONE"
			);
			
			process.stdout.write(
				"Running virtual client... "
			);
			
			this.gameWorld.run();
			
			console.log(
				"DONE"
			);
			
			this.socket.run(
				argv[0] || "127.0.0.1", //Host
				argv[1] || "80" //Port
			);
			
			this.checkPing();
		},
		
		checkPing: function() {
			for (var i in app.sessionStatuses) {
				if (app.sessionStatuses[i]) {
					app.sessionStatuses[i] = false;
				} else {
					app.socket.emit("disconnection", {
						login: i,
						sessionId: app.sessions[i]
					});
				}
			}
			
			setTimeout(app.checkPing, 1000);
		}
};

app.run(process.argv.slice(2));
